@@ -37,9 +37,8 @@ use core::fmt::Debug;
 use raw_window_handle::{RawDisplayHandle, RawWindowHandle};
 use rio_backend::clipboard::Clipboard;
 use rio_backend::clipboard::ClipboardType;
-use rio_backend::config::{
-    colors::term::List,
-    renderer::{Backend as RendererBackend, Performance as RendererPerformance},
+use rio_backend::config::renderer::{
+    Backend as RendererBackend, Performance as RendererPerformance,
 };
 use rio_backend::crosswords::pos::{Boundary, CursorState, Direction, Line};
 use rio_backend::crosswords::search::RegexSearch;
@@ -528,15 +527,6 @@ impl Screen<'_> {
         mode
     }
 
-    #[inline]
-    #[allow(unused)]
-    pub fn colors(&mut self) -> List {
-        let terminal = self.ctx().current().terminal.lock();
-        let mode = terminal.colors();
-        drop(terminal);
-        mode
-    }
-
     #[inline]
     pub fn process_key_event(&mut self, key: &rio_window::event::KeyEvent) {
         if self.context_manager.current().ime.preedit().is_some() {
@@ -1007,6 +997,16 @@ impl Screen<'_> {
                         self.context_manager.select_prev_split();
                         self.render();
                     }
+                    Act::SelectNextSplitOrTab => {
+                        self.cancel_search();
+                        self.context_manager.switch_to_next_split_or_tab();
+                        self.render();
+                    }
+                    Act::SelectPrevSplitOrTab => {
+                        self.cancel_search();
+                        self.context_manager.switch_to_prev_split_or_tab();
+                        self.render();
+                    }
                     Act::SelectTab(tab_index) => {
                         self.context_manager.select_tab(*tab_index);
                         self.cancel_search();
